############################
### Application Results ###
############################
#### Oregon Experiment #####
############################

rm(list=ls())
library(np)
library(MASS)
library(VGAM)
library(xtable)

setwd("C:/Users/Philipp.Bach/Documents/Promotion/DissUndPapers/Econometric Modelling of Count Data (aktueller Stand)/Artikelversion/Artikelversion/Revision/Revision R2/R Code Semiparametric Modeling of Count Data/Application")

set.seed(12345)

# Load TableV Data
data <- read.csv("data_chronic_docvis.csv", header = TRUE, sep=",", dec=".")

n <- nrow(data)

# Merge variables (faster computation - see note in Appendix)
data$ddddraw_sur_comb <- 1*data$ddddraw_sur_2 + 2*data$ddddraw_sur_3 + 3*data$ddddraw_sur_4 + 4*data$ddddraw_sur_5 + 5*data$ddddraw_sur_6 + 6*data$ddddraw_sur_7
data$ddddraXnum_comb <- 1*data$ddddraXnum_2_2 + 2*data$ddddraXnum_2_3 + 3*data$ddddraXnum_3_2 + 4*data$ddddraXnum_3_3 + 5*data$ddddraXnum_4_2 + 6*data$ddddraXnum_5_2 + 7*data$ddddraXnum_6_2+ 8*data$ddddraXnum_7_2 


#######################################################
######## Local Likelihood Estimator ###################
#######################################################

# BW Selection

start.time <- Sys.time()   # measure computation time
start.time

bw <- npregbw(doc_num_mod_12m ~ factor(treatment) + factor(ddddraw_sur_comb) + factor(dddnumhh_li_2) 
              + factor(dddnumhh_li_3) +  factor(ddddraXnum_comb) + factor(female_list) + hhinc_pctfpl_12m 
              + age2008 + factor(edu_12m_2) + factor(edu_12m_3)
              + factor(edu_12m_4) + factor(english_list) + factor(zip_msa) + factor(race_white_12m) + factor(race_black_12m)
              + factor(race_hisp_12m) 
              , bwmethod="cv.ls", bwscaling=FALSE, nmulti=3, data=data)


DT.np <- npreg(bw, regtype="lc", data=data)


end.time <- Sys.time()
time <- end.time - start.time   # compute running time!
time

#save.image("workspace_LLNB_nochronicdis.RData")                       



#########################################################
######### Parametric NB II Models #######################
#########################################################

# Create Variables (interactions)

### Add Interactions for NB2 (para)

data$int1 <- data$hhinc_pctfpl_12m*data$treatment
data$int2 <- (data$hhinc_pctfpl_12m)^2 *data$treatment
data$incsqrt <- data$hhinc_pctfpl_12m*data$hhinc_pctfpl_12m


#############################################################
########## Estimate Parametric Models #######################
#############################################################

# NB2 Regression - Table 10: "Estimates from parametric NB2 regression" 

# column (1) - NB2 version of baseline model from Finkelstein (2012)
nb.reg1 <- vglm(doc_num_mod_12m ~ treatment + ddddraw_sur_comb + dddnumhh_li_2+dddnumhh_li_3 + ddddraXnum_comb
                , family=negbinomial, data=data, weights = data$weights_12m)


# column (2) - extended model (including covariates, linear terms only)
nb.reg2 <- vglm(doc_num_mod_12m ~ treatment + ddddraw_sur_comb + dddnumhh_li_2+dddnumhh_li_3 + ddddraXnum_comb + female_list + hhinc_pctfpl_12m + age2008 + edu_12m_2 + edu_12m_3 + edu_12m_4 + english_list + zip_msa + race_white_12m + race_black_12m + race_hisp_12m
                , family=negbinomial, data=data, weights = data$weights_12m)


###  column (3) - extended model WITH INTERACTION OF INCOME AND TREATMENT VAR.
# With weighting
nb.reg3 <- vglm(doc_num_mod_12m ~ treatment + ddddraw_sur_comb + dddnumhh_li_2+dddnumhh_li_3 + ddddraXnum_comb + female_list + hhinc_pctfpl_12m + age2008 + edu_12m_2 + edu_12m_3 + edu_12m_4 + english_list + zip_msa + race_white_12m + race_black_12m + race_hisp_12m
                + int1, negbinomial  , data=data, weights = data$weights_12m)


#  column (4) - extended model WITH INTERACTION OF TREATMENT VAR WITH ALL REGRESSORS (INCLUDING INCOME SQUARED)
nb.reg4 <- vglm(doc_num_mod_12m ~ treatment + ddddraw_sur_comb + dddnumhh_li_2+ dddnumhh_li_3 + ddddraXnum_comb 
                +  female_list + hhinc_pctfpl_12m + age2008 + edu_12m_2 + edu_12m_3 + edu_12m_4 + english_list 
                + zip_msa + race_white_12m + race_black_12m + race_hisp_12m
                + incsqrt 
                + treatment:(female_list + hhinc_pctfpl_12m + age2008 + edu_12m_2 + edu_12m_3 + edu_12m_4 + english_list 
                             + zip_msa + race_white_12m + race_black_12m + race_hisp_12m
                             + incsqrt)
                , negbinomial  , data=data, weights = data$weights_12m)



#############################################################
######### Estimate Incremental Effects ######################
#############################################################

### LLNB 

# Predict hypothetical values Xd=1 for every observation
newdata.1 <- data.frame(data)
newdata.1$treatment <- 1 
newdata.1$int1 <- newdata.1$hhinc_pctfpl_12m*newdata.1$treatment
newdata.1$int2 <- (newdata.1$hhinc_pctfpl_12m)^2*newdata.1$treatment


# Predict hypothetical values Xd=0 for every observation
newdata.0 <- data.frame(data)
newdata.0$treatment <- 0 
newdata.0$int1 <- newdata.0$hhinc_pctfpl_12m*newdata.0$treatment
newdata.0$int2 <- (newdata.0$hhinc_pctfpl_12m)^2*newdata.0$treatment

# 
pred.1 <- predict(DT.np, newdata = newdata.1, type="response")
pred.0 <- predict(DT.np, newdata = newdata.0, type="response")

data$ME_pred <- pred.1 - pred.0


###############################################################################################
################# ESTIMATION OF INCREMENTAL EFFECTS / -> BOXPLOTS #############################
###############################################################################################

##### Specification in column (2)
# Generate Predictions
pred.pnb2.1 <- predict(nb.reg2, newdata = newdata.1, type="response")
pred.pnb2.0 <- predict(nb.reg2, newdata = newdata.0, type="response")

# IEs
ME.pnb2_pred <- pred.pnb2.1 - pred.pnb2.0

##### 2) Specification in column (3) 
pred.nb3.1 <- predict(nb.reg3, newdata=newdata.1, type="response")
pred.nb3.0 <- predict(nb.reg3, newdata=newdata.0, type="response")

ME.pnb3_pred <- pred.nb3.1 - pred.nb3.0


##### 3) Specification in column (4) 
pred.nb4.1 <- predict(nb.reg4, newdata=newdata.1, type="response")
pred.nb4.0 <- predict(nb.reg4, newdata=newdata.0, type="response")

ME.pnb4_pred <- pred.nb4.1 - pred.nb4.0


################################################################
##### Plots and Boxplots #######################################
################################################################



########################################################################
######## INCOME #########################################################
#########################################################################

# Boxplots

# LLNB
boxplot( data$ME_pred[data$hhinc_pctfpl_12m >= 0 & data$hhinc_pctfpl_12m < 51], # 0-50
         data$ME_pred[data$hhinc_pctfpl_12m > 50 & data$hhinc_pctfpl_12m < 101], # 51 - 100
         data$ME_pred[data$hhinc_pctfpl_12m > 100 & data$hhinc_pctfpl_12m < 151], # 101-150
         data$ME_pred[data$hhinc_pctfpl_12m > 150 & data$hhinc_pctfpl_12m < 201], # 151-200
         data$ME_pred[data$hhinc_pctfpl_12m > 200 & data$hhinc_pctfpl_12m < 251], # 201-250
         data$ME_pred[data$hhinc_pctfpl_12m > 250 & data$hhinc_pctfpl_12m < 301],  # 250-300
         data$ME_pred[data$hhinc_pctfpl_12m > 300 & data$hhinc_pctfpl_12m < 351],  # 301-350
         data$ME_pred[data$hhinc_pctfpl_12m > 350 & data$hhinc_pctfpl_12m < 401],  # 351-400
         data$ME_pred[data$hhinc_pctfpl_12m > 400], # 400+
         ylim=c(-1.9,2.31), 
         ylab = "ITT", xlab = "Income (% of federal poverty level)", main = "", cex.lab = 1.5,
         names = c("0-50","51-100","101-150","151-200", "201-250", "251-300", "301-350", "351-400", "400+"))#, cex.lab = 1.5)
abline(a = 0, b = 0, lty=3)

# PNB
boxplot( ME.pnb2_pred[data$hhinc_pctfpl_12m >= 0 & data$hhinc_pctfpl_12m < 51], # 0-50
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 50 & data$hhinc_pctfpl_12m < 101], # 51 - 100
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 100 & data$hhinc_pctfpl_12m < 151], # 101-150
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 150 & data$hhinc_pctfpl_12m < 201], # 151-200
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 200 & data$hhinc_pctfpl_12m < 251], # 201-250
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 250 & data$hhinc_pctfpl_12m < 301],  # 250-300
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 300 & data$hhinc_pctfpl_12m < 351],  # 301-350
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 350 & data$hhinc_pctfpl_12m < 401],  # 351-400
         ME.pnb2_pred[data$hhinc_pctfpl_12m > 400], # 400+
         ylim=c(-1.9,2.31), 
         ylab = "ITT", xlab = "Income (% of federal poverty level)", main = "", cex.lab = 1.5,
         names = c("0-50","51-100","101-150","151-200", "201-250", "251-300", "301-350", "351-400", "400+"))#, cex.lab = 1.5)
abline(a = 0, b = 0, lty=3)

# PNB
boxplot( ME.pnb3_pred[data$hhinc_pctfpl_12m >= 0 & data$hhinc_pctfpl_12m < 51], # 0-50
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 50 & data$hhinc_pctfpl_12m < 101], # 51 - 100
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 100 & data$hhinc_pctfpl_12m < 151], # 101-150
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 150 & data$hhinc_pctfpl_12m < 201], # 151-200
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 200 & data$hhinc_pctfpl_12m < 251], # 201-250
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 250 & data$hhinc_pctfpl_12m < 301],  # 250-300
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 300 & data$hhinc_pctfpl_12m < 351],  # 301-350
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 350 & data$hhinc_pctfpl_12m < 401],  # 351-400
         ME.pnb3_pred[data$hhinc_pctfpl_12m > 400], # 400+
         ylim=c(-1.9,2.31), 
         ylab = "ITT", xlab = "Income (% of federal poverty level)", main = "", cex.lab = 1.5,
         names = c("0-50","51-100","101-150","151-200", "201-250", "251-300", "301-350", "351-400", "400+"))#, cex.lab = 1.5)
abline(a = 0, b = 0, lty=3)

boxplot( ME.pnb4_pred[data$hhinc_pctfpl_12m >= 0 & data$hhinc_pctfpl_12m < 51], # 0-50
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 50 & data$hhinc_pctfpl_12m < 101], # 51 - 100
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 100 & data$hhinc_pctfpl_12m < 151], # 101-150
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 150 & data$hhinc_pctfpl_12m < 201], # 151-200
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 200 & data$hhinc_pctfpl_12m < 251], # 201-250
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 250 & data$hhinc_pctfpl_12m < 301],  # 250-300
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 300 & data$hhinc_pctfpl_12m < 351],  # 301-350
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 350 & data$hhinc_pctfpl_12m < 401],  # 351-400
         ME.pnb4_pred[data$hhinc_pctfpl_12m > 400], # 400+
         ylim=c(-1.9,2.31), 
         ylab = "ITT", xlab = "Income (% of federal poverty level)", main = "", cex.lab = 1.5,
         names = c("0-50","51-100","101-150","151-200", "201-250", "251-300", "301-350", "351-400", "400+"))#, cex.lab = 1.5)
abline(a = 0, b = 0, lty=3)




###########################################################
#### Model Fit ############################################
###########################################################

#### LLNB

pred.np <- predict(DT.np, type="response", data=data)

MSE_np <- 1/n *sum((data$doc_num_mod_12m - pred.np)^2)
MAE_np <- 1/n *sum(abs(data$doc_num_mod_12m - pred.np))
RMSE_np <-sqrt(1/n * sum((data$doc_num_mod_12m - pred.np)^2))

res.np <- cbind(MSE_np, MAE_np, RMSE_np)
colnames(res.np) <- c("MSE", "MAE", "RMSE")
rownames(res.np) <- "LLNB"

res.np


#### Linearly specified para2metric Model
pred.para2 <- predict(nb.reg2, type="response", data=data)

MSE_para2 <- 1/n *sum((data$doc_num_mod_12m - pred.para2)^2)
MAE_para2 <- 1/n *sum(abs(data$doc_num_mod_12m - pred.para2))
RMSE_para2 <-sqrt(1/n * sum((data$doc_num_mod_12m - pred.para2)^2))

res.para2 <- cbind(MSE_para2, MAE_para2, RMSE_para2)
colnames(res.para2) <- c("MSE", "MAE", "RMSE")
rownames(res.para2) <- "PNB"

res.para2

#### Linearly specified parametric model with interaction
pred.para3 <- predict(nb.reg3, type="response", data=data)
MSE_para3 <- 1/n *sum((data$doc_num_mod_12m - pred.para3)^2)
MAE_para3 <- 1/n *sum(abs(data$doc_num_mod_12m - pred.para3))
RMSE_para3 <-sqrt(1/n * sum((data$doc_num_mod_12m - pred.para3)^2))

res.para3 <- cbind(MSE_para3, MAE_para3, RMSE_para3)
colnames(res.para3) <- c("MSE", "MAE", "RMSE")
rownames(res.para3) <- "PNB"

res.para3


##### With income categories and interactions with treatment 
pred.para4 <- predict(nb.reg4, type="response", data=data)
MSE_para4 <- 1/n *sum((data$doc_num_mod_12m - pred.para4)^2)
MAE_para4 <- 1/n *sum(abs(data$doc_num_mod_12m - pred.para4))
RMSE_para4 <-sqrt(1/n * sum((data$doc_num_mod_12m - pred.para4)^2))

res.para4 <- cbind(MSE_para4, MAE_para4, RMSE_para4)
colnames(res.para4) <- c("MSE", "MAE", "RMSE")
rownames(res.para4) <- "PNB"

res.para4





