
################################################## 
################ Simulation Study ################ 
##################################################

################ 
## DGPs 1 - 4 ##
################

## Setting up the DGPs as functions, n sample size
# First restrict attention to: 
# DGP1 : NB II linear 
# DGP2 : NB II nonlinear 
# DGP3 : Zero-inflated NB II linear
# DGP4 : Zero-inflated NB II nonlinear

DGP1 <- function(n) {
  Xi_c1 <- runif(n, min = 0, max = 1)
  Xi_d1 <- sample(0:1, size=n, replace=TRUE, prob=rep(1/2,2))
  Xi_d2 <- sample(0:2, size=n, replace=TRUE, prob=rep(1/3,3))
  mu <- exp(1.2 - 0.4*Xi_c1  + 0.5*Xi_d1 - 0.8*Xi_d2)
  size <- 7 # size parameter (sigma^(-2))
  Y <- rnbinom(n, size=size, mu=mu)
  return(as.data.frame(cbind(Y,mu, size, Xi_c1, Xi_d1, Xi_d2)))
}

DGP2 <- function(n) {
  Xi_c1 <- runif(n, min=0, max=1)
  Xi_d1 <- sample(0:1, size=n, replace=TRUE, prob=rep(1/2,2))
  Xi_d2 <- sample(0:2, size=n, replace=TRUE, prob=rep(1/3,3))
  mu <- exp(0.8 + 2.5*Xi_c1 - 2.8*Xi_c1^2 + 0.5*Xi_d1 - 0.1*Xi_d2 + 0.8*Xi_c1*Xi_d1 + 1.2*Xi_d2*Xi_c1 - 1*Xi_d2^2)
  size <- 7   # size parameter (sigma^(-2))
  Y <- rnbinom(n, size=size, mu=mu)
  return(as.data.frame(cbind(Y,mu, size, Xi_c1, Xi_d1, Xi_d2)))
}

DGP3 <- function(n) {
  Xi_c1 <- runif(n, min=0, max=1)
  Xi_d1 <- sample(0:1, size=n, replace=TRUE, prob=rep(1/2,2))
  Xi_d2 <- sample(0:2, size=n, replace=TRUE, prob=rep(1/3,3))
  mu <- exp(1.2 - 0.4*Xi_c1  + 0.5*Xi_d1 - 0.8*Xi_d2)
  size <- 7   # size parameter (sigma^(-2))
  p <- 0.2
  zinf <- rbinom(n, size = 1, prob = p) 
  Y <- ifelse(zinf > 0, 0, rnbinom(n, size=size, mu=mu))
  mu <- ifelse(zinf > 0, 0, mu)
    return(as.data.frame(cbind(Y,mu, size, Xi_c1, Xi_d1, Xi_d2, p, zinf)))
}

DGP4 <- function(n) {
  Xi_c1 <- runif(n, min=0, max=1)
  Xi_d1 <- sample(0:1, size=n, replace=TRUE, prob=rep(1/2,2))
  Xi_d2 <- sample(0:2, size=n, replace=TRUE, prob=rep(1/3,3))
  mu <- exp(0.8 + 2.5*Xi_c1 - 2.8*Xi_c1^2 + 0.5*Xi_d1 - 0.1*Xi_d2 + 0.8*Xi_c1*Xi_d1 + 1.2*Xi_d2*Xi_c1 - 1*Xi_d2^2)
  size <- 7   # size parameter (sigma^(-2))
  p <- 0.2
  zinf <- rbinom(n, size = 1, prob = p) 
  Y <- ifelse(zinf > 0, 0, rnbinom(n, size=size, mu=mu))
  mu <- ifelse(zinf > 0, 0, mu)
  return(as.data.frame(cbind(Y,mu, size, Xi_c1, Xi_d1, Xi_d2, p, zinf)))
}


