rm(list=ls())
library(np)
library(VGAM)

setwd("C:/Users/Philipp.Bach/Documents/Promotion/DissUndPapers/Econometric Modelling of Count Data (aktueller Stand)/Artikelversion/Artikelversion/Revision/Revision R2/R Code Semiparametric Modeling of Count Data/Simulation")
source("DGPs.R")
source("functions.R")

set.seed(1234)

R <- 500 # number of Repetitions
n <- c(400) # in-sample size (training sample)

formula1 <- formula("Y ~ Xi_c1 + Xi_d1 + Xi_d2") # regression formula baseline NB2 model (PNB 1)
formula7 <- formula("Y ~ Xi_c1 + Xi_d1 + Xi_d2 + Xi_c1*Xi_d1 + Xi_c1*Xi_d2 + Xi_d1*Xi_d2 + Xi_c12") # (PNB 2)

# Grid to construct benchmark observations (Xi_d2 = 0, Xi_c1 = xval, Xi_d1 = 0 or 1 respectively)
xval <- seq(0,1,0.001)

### Objects to save results: 

# Object to save Marginal Effects from every repetition
# All Marginal Effects
ME <- array(NA, dim = c(length(xval), 10 ,R))

# Save marginal effect at the mean (of Xi_c1) in array
ME.atmean <- array(NA, dim=c(length(n), 10 ,R))

# Average treatment effect 
AME  <- array(NA, dim=c(length(n), 10 ,R))

## MSE
# Average Marginal Effect (absolute)
intermediate_res.MSE.AME <- array(NA, dim=c(length(n), 9, R))
# Marginal Effect at the Mean
intermediate_res.MSE.MEaM <- array(NA, dim=c(length(n), 9, R))
# "Integrated" MSE, all MEs
intermediate_res.MSE.ME <- array(NA, dim=c(length(n), 9, R))

## MAE
# Average Marginal Effect (absolute)
intermediate_res.MAE.AME <- array(NA, dim=c(length(n), 9, R))
# Marginal Effect at the Mean
intermediate_res.MAE.MEaM <- array(NA, dim=c(length(n), 9, R))
# "Integrated" MAE, all MEs
intermediate_res.MAE.ME <- array(NA, dim=c(length(n), 9, R))

# Save bandwidths of LLNB and NPCDE
bw.object.DGP1 <- array(NA, dim=c(2,4,length(n), R))


### Start Loop 

# initial index for seed (in loop)
ind <- 12345

start.time <- Sys.time()   # measure computation time
start.time

for (i in 1:length(n)) { # loop over sample sizes , in- and out-of-sample

  s <- n[i]  # s = in-sample size in loop (50, 100, 200)
  
  for (j in 1:R) { # loop over the repetitions
    
    ind <- ind + 1  # running seed for replicability
    set.seed(ind)


data <- DGP1(s) # Generate data set (100, 200, 400)
data$Xi_c12 <- (data$Xi_c1)^2

data_ins <- data[1:s,] # Generate training sample

# Construct data to generate predictions upon (grid of Xi_c1) 

###############
## Xi_d1 = 0 ##
############### 

# True effect (oracle) according to DGP1 (no estimation error!)
# Xi_d1 = 1/0; Xi_d2 = 0
mu1 <- exp(1.2 - 0.4*xval  + 0.5)
mu0 <- exp(1.2 - 0.4*xval)

mu1.atmean <- exp(1.2 - 0.4*mean(xval) + 0.5)
mu0.atmean <-  exp(1.2 - 0.4*mean(xval))

  # save True Marginal Effect (oracle)
  ME[,1,j] <- mu1 - mu0

  ME.true <- mu1 - mu0

  AME[i,1,j] <- mean(mu1 - mu0)

  ME.true.atmean <- mu1.atmean - mu0.atmean

  ME.atmean[i,1,j] <- ME.true.atmean 
 
              
####################################
### "Newdata" for predictions of ###
## PNBs/LLNB/NPCDE, based on grid ##
####################################
  
newx1 <- as.data.frame(cbind(Xi_c1=xval, Xi_d1=1, Xi_d2=0, Xi_c12 = (xval)^2))
newx0 <- as.data.frame(cbind(Xi_c1=xval, Xi_d1=0, Xi_d2=0, Xi_c12 = (xval)^2))

#### Fit Parametric NB models: 

 # PNB 1 (baseline)

  try({
  nb.reg <-vglm(formula1, negbinomial, data=data_ins) # Code for NB2 (unknown, constant sigma^(-2))
  # Predict for every gridpoint 
  pred1 <- predict(nb.reg, type="response", newdata=newx1)
  pred0 <- predict(nb.reg, type="response", newdata=newx0)

  # Predict "at the mean" (of Xi_c1)
  pred1.atmean <- pred1[(length(xval)+1)/2]
  # pred0.atmean <- predict(nb.reg, type = "response", newdata = newx0[(length(xval)+1)/2,])
  pred0.atmean <- pred0[(length(xval)+1)/2]
  
  # Marginal Effects
  ME.PNB <- pred1 - pred0
 
   ME[,2,j] <- ME.PNB
 
  # Average Marginal Effect
  AME[i,2,j] <- mean(ME.PNB)
  
  intermediate_res.MSE.AME[i,1,j] <- (mean(ME.true) - mean(ME.PNB))^2
  intermediate_res.MAE.AME[i,1,j] <- abs(mean(ME.true) - mean(ME.PNB))
  
  # Integrated MSE/MAE for (r)MEs
  intermediate_res.MSE.ME[i,1,j] <- mean((ME.true - ME.PNB)^2)
  intermediate_res.MAE.ME[i,1,j] <- mean(abs(ME.true - ME.PNB))
  
  # ME at the Mean 
  ME.PNB.atmean <- pred1.atmean - pred0.atmean
  ME.atmean[i,2,j] <- ME.PNB.atmean
 
  intermediate_res.MSE.MEaM[i,1,j] <- (ME.true.atmean  - ME.PNB.atmean)^2
  intermediate_res.MAE.MEaM[i,1,j] <- abs(ME.true.atmean  - ME.PNB.atmean)

  rm(pred1, pred0)
  rm(pred1.atmean, pred0.atmean)
  rm(ME.PNB)
  rm(ME.PNB.atmean)
  }, FALSE)
  
  # PNB 2 (fully saturated with quadratic interaction)

try({
  nb.reg7 <-vglm(formula7, negbinomial, data=data_ins) # Code for NB2 (unknown, constant sigma^(-2))
  # Predict for every gridpoint 
  pred1 <- predict(nb.reg7, type="response", newdata=newx1)
  pred0 <- predict(nb.reg7, type="response", newdata=newx0)
  # Predict "at the mean" (of Xi_c1)
  #pred1.atmean <- predict(nb.reg, type = "response", newdata = newx1[(length(xval)+1)/2,])
  # faster (identical output)
  pred1.atmean <- pred1[(length(xval)+1)/2]
  # pred0.atmean <- predict(nb.reg, type = "response", newdata = newx0[(length(xval)+1)/2,])
  pred0.atmean <- pred0[(length(xval)+1)/2]
  
  ME.PNB <- pred1 - pred0

  # Marginal Effects 
  ME[,8,j] <-  ME.PNB
 
  # Average Marginal Effect
  AME[i,8,j] <- mean(ME.PNB)
  
  intermediate_res.MSE.AME[i,7,j] <- (mean(ME.true) - mean(ME.PNB))^2
  intermediate_res.MAE.AME[i,7,j] <- abs(mean(ME.true) - mean(ME.PNB))
  
  # Integrated MSE/MAE for MEs
  intermediate_res.MSE.ME[i,7,j] <- mean((ME.true - ME.PNB)^2)
  intermediate_res.MAE.ME[i,7,j] <- mean(abs(ME.true - ME.PNB))
  
  # ME at the Mean 
  ME.PNB.atmean <- pred1.atmean - pred0.atmean
  ME.atmean[i,8,j] <- ME.PNB.atmean
 
  intermediate_res.MSE.MEaM[i,7,j] <- (ME.true.atmean  - ME.PNB.atmean)^2
  intermediate_res.MAE.MEaM[i,7,j] <- abs(ME.true.atmean  - ME.PNB.atmean)
 
  rm(pred1, pred0)
  rm(pred1.atmean, pred0.atmean)
  rm(ME.PNB)
  rm(ME.PNB.atmean)
}, FALSE)

##### Fit Semiparametric NB model: 

  # NB 1 (baseline)
  bw.nb <- npregbw(Y ~ Xi_c1 + factor(Xi_d1) + factor(Xi_d2), regtype="lc", bwmethod = "cv.ls", data=data_ins)  
  nb.np <- npreg(bws = bw.nb)
  
  bw.object.DGP1[1,-1,i,j] <- bw.nb$bw
  
  ### LLNB estimate, based on grid
  pred.llnb1 <- predict(nb.np, type="response", newdata=newx1)
  pred.llnb0 <- predict(nb.np, type="response", newdata=newx0)
  
  # Predict "at the mean" (of Xi_c1)
  pred.llnb1.atmean <- pred.llnb1[(length(xval)+1)/2]
  pred.llnb0.atmean <- pred.llnb0[(length(xval)+1)/2]
  
  # Marginal Effects
  ME.LLNB <- pred.llnb1 - pred.llnb0 
  
  ME[,9,j] <- ME.LLNB

    # Average Marginal Effect
  AME[i,9,j] <- mean(ME.LLNB)

  intermediate_res.MSE.AME[i,8,j] <- (mean(ME.true) - mean(ME.LLNB))^2
  intermediate_res.MAE.AME[i,8,j] <- abs(mean(ME.true) - mean(ME.LLNB))

  # Integrated MSE/MAE for (r)MEs
  intermediate_res.MSE.ME[i,8,j] <- mean((ME.true - ME.LLNB)^2)
  intermediate_res.MAE.ME[i,8,j] <- mean(abs(ME.true - ME.LLNB))
  
  # ME at the Mean 
  ME.LLNB.atmean <- pred.llnb1.atmean - pred.llnb0.atmean
  ME.atmean[i,9,j] <- ME.LLNB.atmean

  intermediate_res.MSE.MEaM[i,8,j] <- (ME.true.atmean  - ME.LLNB.atmean)^2
  intermediate_res.MAE.MEaM[i,8,j] <- abs(ME.true.atmean  - ME.LLNB.atmean)
  
  
  rm(pred.llnb1, pred.llnb0)
  rm(pred.llnb1.atmean, pred.llnb0.atmean)
  rm(ME.LLNB)
  rm(ME.LLNB.atmean)


##### NPCDE: 
  bw.dens <- npcdensbw(ordered(Y) ~ Xi_c1 + factor(Xi_d1) + factor(Xi_d2), regtype="lc", bwmethod = "cv.ls", data=data_ins) 
  fully_np <- npcdens(bws=bw.dens)
  
  bw.object.DGP1[2,-1,i,j] <- bw.dens$xbw
  bw.object.DGP1[2,1,i,j] <- bw.dens$ybw
  
  val <- 0:(max(data$Y)+5)
  pred.npcde1 <- predict_modal_dens(fully_np, newx1, range=val)  # predict modal value of distribution!
  pred.npcde0 <- predict_modal_dens(fully_np, newx0, range=val) 
  
  
  # Marginal Effects
  ME.NPCDE <- pred.npcde1 - pred.npcde0 
 
  # Predict "at the mean" (of Xi_c1)
  pred.npcde1.atmean <- pred.npcde1[(length(xval)+1)/2]
  pred.npcde0.atmean <- pred.npcde0[(length(xval)+1)/2]
  
  ME[,10,j] <- ME.NPCDE
  
  # Average Marginal Effect
  AME[i,10,j] <- mean(ME.NPCDE)
  
  intermediate_res.MSE.AME[i,9,j] <- (mean(ME.true) - mean(ME.NPCDE))^2
  intermediate_res.MAE.AME[i,9,j] <- abs(mean(ME.true) - mean(ME.NPCDE))
 
  # Integrated MSE/MAE for (r)MEs
  intermediate_res.MSE.ME[i,9,j] <- mean((ME.true - ME.NPCDE)^2)
  intermediate_res.MAE.ME[i,9,j] <- mean(abs(ME.true - ME.NPCDE))

  # ME at the Mean 
  ME.NPCDE.atmean <- pred.npcde1.atmean - pred.npcde0.atmean
  ME.atmean[i,10,j] <- ME.NPCDE.atmean

  intermediate_res.MSE.MEaM[i,9,j] <- (ME.true.atmean  - ME.NPCDE.atmean)^2
  intermediate_res.MAE.MEaM[i,9,j] <- abs(ME.true.atmean  - ME.NPCDE.atmean)
  
  rm(pred.npcde1, pred.npcde0)
  rm(pred.npcde1.atmean, pred.npcde0.atmean)
  rm(ME.NPCDE)
  rm(ME.NPCDE.atmean)
  
  
  }
}
  

end.time <- Sys.time()
time <- end.time - start.time   # compute running time!
time
  
  
##############################################
### Aggregate (intermed.) Results ############
##############################################

### Results 
results.MSE.AME <- mean_array(intermediate_res.MSE.AME)
results.MSE.MEaM <- mean_array(intermediate_res.MSE.MEaM)
results.MSE.ME <- mean_array(intermediate_res.MSE.ME)

results.MAE.AME <- mean_array(intermediate_res.MAE.AME)
results.MAE.MEaM <- mean_array(intermediate_res.MAE.MEaM)
results.MAE.ME <- mean_array(intermediate_res.MAE.ME)

### Names for result objects
colnames(results.MSE.AME)  <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.MSE.MEaM) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.MSE.ME)   <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")

colnames(results.MAE.AME) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.MAE.MEaM) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.MAE.ME) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")


results.MSE.AME 
results.MSE.MEaM 
results.MSE.ME 

results.MAE.AME 
results.MAE.MEaM 
results.MAE.ME 


###################### 
### PLOTS ############
######################

# Rep 1 and 2

plot(ME[,1,1] ~xval, ylim=c(0,8), main = "Abs. MEs (1st Rep)")
points(ME[,2,1] ~xval, col = "green")
points(ME[,8,1] ~xval, col = "red")
points(ME[,9,1] ~xval, col = "darkblue")
legend("topleft", col = c("black", "green", "red", "darkblue"), lty = c(1,1,1,1), legend = c("TRUE", "PNB1", "PNB2", "LLNB"))

plot(ME[,1,2] ~xval, ylim=c(0,8), main = "Abs. MEs (2nd Rep)")
points(ME[,2,2] ~xval, col = "green")
points(ME[,8,2] ~xval, col = "red")
points(ME[,9,2] ~xval, col = "darkblue")
legend("topleft", col = c("black", "green", "red", "darkblue"), lty = c(1,1,1,1), legend = c("TRUE", "PNB1", "PNB2", "LLNB"))

# Boxplot all MEs (over all repetitions)
boxplot(ME[,1,], ylim = c(0,12))
boxplot(ME[,2,], ylim = c(0,12))
boxplot(ME[,8,], ylim = c(0,12))
boxplot(ME[,9,], ylim = c(0,12))

# Boxplot all AMEs (over all reps)
boxplot(AME[1,1,], AME[1,2,], AME[1,8,], AME[1,9,] )

### Plot mean marginal effect predicted for every grid point over all repetitions
mean.ME.PNB <- apply(ME[,2,],1, mean)
mean.ME.PNB2 <- apply(ME[,8,],1, mean)
mean.ME.LLNB <- apply(ME[,9,],1, mean)
mean.ME.NPCDE <- apply(ME[,10,],1, mean)

plot(ME[,1,1] ~xval, main = "", ylim = c(0.8,8), col = "grey", type = "l", lty = 1, lwd = 6, 
     ylab = expression("Incremental Effect of X"[d1]), xlab = expression("X"[c1]))
points(mean.ME.PNB ~ xval, col = "black", type = "l", lty = 1, lwd = 2)
points(mean.ME.PNB2 ~ xval, type = "l", lty = 4, lwd = 2)
points(mean.ME.LLNB ~ xval, col = "black", type = "l", lty = 5, lwd = 2)
points(mean.ME.NPCDE ~ xval,  type = "l", lty = 3, lwd = 2.5)
legend("topleft", col = c("grey", "black", "black", "black", "black"), lty = c(1,1,4,5,3), lwd = c(4,2,2,2,3), 
       legend = c("TRUE", "PNB (1)", "PNB (2)", "LLNB", "NPCDE"), cex = 1.3)

##############################
#### Save IMSE and IMAE ######
##############################
save(results.MSE.ME, file = "resultsMSE_DGP1.RData")
save(results.MAE.ME, file = "resultsMAE_DGP1.RData")

save.image("workspace_treatmentsimulation_DGP1_d2-0.RData")
