################################################## 
################ Simulation Study ################ 
##################################################

##################################################
##############  Revision  ########################
##################################################

################ 
#### DGP1 ######
################ 

rm(list=ls())
setwd("C:/Users/Philipp.Bach/Documents/Promotion/DissUndPapers/Econometric Modelling of Count Data (aktueller Stand)/Artikelversion/Artikelversion/Revision/Revision R2/R Code Semiparametric Modeling of Count Data/Simulation")
library(np)
library(VGAM) # for NB2 regression
library(stargazer) # for output (to LaTex)

source("DGPs.R")
source("functions.R")

# Turn all warning messages into errors, necessary for try() command
options(warn=2)

## set up
R <- 500 # number of Repetitions
n <- c(50, 100, 200) # in-sample size (training sample)
N <- c(50, 100, 200) # out-of-sample size (validation sample)
# => total sample size (100, 200, 400)
### Use "validation set approach", 50% sample split (James et al., p.180ff)

# PNB 1
formula1 <- formula("Y ~ Xi_c1 + Xi_d1 + Xi_d2") # regression formula baseline NB2 model 

# PNB 2
formula7 <- formula("Y ~ Xi_c1 + Xi_d1 + Xi_d2 + Xi_c1*Xi_d1 + Xi_c1*Xi_d2 + Xi_d1*Xi_d2 + Xi_c12") #fully saturated model with quadratic term              

# Names in result matrices: "PNB1", "PNB2", "LLNB", "NPCDE"

# Keep only those results that are necessary for revision (oracle and bias)!

# Oracle MSE and MAE (mu)
results.oMSE <- matrix(NA, nrow=length(n), ncol=9)
results.oMAE <- matrix(NA, nrow=length(n), ncol=9)
results.oRMSE <- matrix(NA, nrow=length(n), ncol=9)

# Oracle MSE and MAE (sigma)
results.oMSE.sigma <- matrix(NA, nrow=length(n), ncol=9)
results.oMAE.sigma <- matrix(NA, nrow=length(n), ncol=9)
results.oRMSE.sigma <- matrix(NA, nrow=length(n), ncol=9)

# Bias
results.bias.y <- matrix(NA, nrow=length(n), ncol=9)
results.bias.mu <- matrix(NA, nrow=length(n), ncol=9)
results.bias.sigma <- matrix(NA, nrow=length(n), ncol=9)

# Create arrays to store intermediate results, i.e. results obtained in every repetition

# oracle MSE and MAE (mu)
intermediate_res.oMSE <- array(NA, dim=c(length(n), 9, R))
intermediate_res.oMAE <- array(NA, dim=c(length(n), 9, R))
intermediate_res.oRMSE <- array(NA, dim=c(length(n), 9, R))

# oracle MSE and MAE(sigma)
intermediate_res.oMSE.sigma <- array(NA, dim=c(length(n), 9, R))
intermediate_res.oMAE.sigma <- array(NA, dim=c(length(n), 9, R))
intermediate_res.oRMSE.sigma <- array(NA, dim=c(length(n), 9, R))

# Bias
intermediate_res.bias.y <- array(NA, dim=c(length(n), 9, R))
intermediate_res.bias.mu <- array(NA, dim=c(length(n), 9, R))
intermediate_res.bias.sigma <- array(NA, dim=c(length(n), 9, R))

# Save bandwidths of LLP, LLNB and NPCDE
bw.object.DGP1 <- array(NA, dim=c(2,4,length(n), R))

# initial index for seed (in loop)
ind <- 12345

start.time <- Sys.time()   # measure computation time
start.time

for (i in 1:length(n)) { # loop over sample sizes , in- and out-of-sample
  s <- n[i]  # s = in-sample size in loop (50, 100, 200)
  q <- N[i]  # q = out-of-sample size in loop (50, 100, 200)
  
  for (j in 1:R) { # loop over the repetitions
    
    ind <- ind + 1  # running seed for replicability
    set.seed(ind)
    
    data <- DGP1(s+q) # Generate data set (100, 200, 400)
    data$Xi_c12 <- (data$Xi_c1)^2
    data_ins <- data[1:s,] # Generate training sample
    data_oos <- data[(s+1):(s+q),] # Generate validation sample

    
    # Parametric NB 1 (PNB 1) - Linear Specification
    
    try({ # Use try command in order to exclude results of repetitions in which nb2 estimator does not converge!
      nb.reg <-vglm(formula1, negbinomial, data=data_ins) # Code for NB2 (unknown, constant sigma^(-2))
      pred <- predict(nb.reg, type="response", newdata=data_oos)
      
      # oracle MSE 
      intermediate_res.oMSE[i,1,j] <- 1/q*sum((data_oos$mu-pred)^2)
      # oracle MAE
      intermediate_res.oMAE[i,1,j] <- 1/q*sum(abs((data_oos$mu-pred)))
      # oracle RMSE
      intermediate_res.oRMSE[i,1,j] <- sqrt(1/q*sum((data_oos$mu-pred)^2))
      
      # oracle MSE, MAE and RMSE (sigma)
      intermediate_res.oMSE.sigma[i,1,j] <- 1/q*sum((data_oos$size - exp(predict(nb.reg)[,2]))^2)
      
      ###intermediate_res.oRMSE.sigma[i,2,j] <- 1/q*sqrt(sum((data_oos$size - exp(predict(nb.reg)[,2]))^2))
      intermediate_res.oRMSE.sigma[i,1,j] <- sqrt(1/q*sum((data_oos$size - exp(predict(nb.reg)[,2]))^2))
      intermediate_res.oMAE.sigma[i,1,j] <- 1/q*sum(abs(data_oos$size - exp(predict(nb.reg)[,2])))
      
      # Bias
      intermediate_res.bias.y[i,1,j] <- 1/q*sum(pred - data_oos$Y) # observed
      intermediate_res.bias.mu[i,1,j] <- 1/q*sum(pred - data_oos$mu) # oracle
      intermediate_res.bias.sigma[i,1,j] <- 1/q*sum(exp(predict(nb.reg)[,2]) - data_oos$size ) # oracle
      
    
      rm(pred)
     
      
    }, FALSE) #FALSE: show error messages!
    
    
    # Parametric NB 2 (PNB 2) - Fully Saturated Model with quadratic term
    
    try({ # Use try command in order to exclude results of repetitions in which nb2 estimator does not converge!
      nb.reg2 <-vglm(formula7, negbinomial, data=data_ins) # Code for NB2 (unknown, constant sigma^(-2))
      pred <- predict(nb.reg2, type="response", newdata=data_oos)
      
      # oracle MSE 
      intermediate_res.oMSE[i,7,j] <- 1/q*sum((data_oos$mu-pred)^2)
      # oracle MAE
      intermediate_res.oMAE[i,7,j] <- 1/q*sum(abs((data_oos$mu-pred)))
      # oracle RMSE
      intermediate_res.oRMSE[i,7,j] <- sqrt(1/q*sum((data_oos$mu-pred)^2))
      
      # oracle MSE, MAE and RMSE (sigma)
      intermediate_res.oMSE.sigma[i,7,j] <- 1/q*sum((data_oos$size - exp(predict(nb.reg2)[,2]))^2)
      intermediate_res.oRMSE.sigma[i,7,j] <- sqrt(1/q*sum((data_oos$size - exp(predict(nb.reg2)[,2]))^2))
      intermediate_res.oMAE.sigma[i,7,j] <- 1/q*sum(abs(data_oos$size - exp(predict(nb.reg2)[,2])))
      
      # Bias
      intermediate_res.bias.y[i,7,j] <- 1/q*sum(pred - data_oos$Y) # observed
      intermediate_res.bias.mu[i,7,j] <- 1/q*sum(pred - data_oos$mu) # oracle
      intermediate_res.bias.sigma[i,7,j] <- 1/q*sum(exp(predict(nb.reg2)[,2]) - data_oos$size ) # oracle
      
      
      rm(pred)
      
      
    }, FALSE) #FALSE: show error messages!
    
    #  Local likelihood NB
    
    bw.nb <- npregbw(Y ~ Xi_c1 + factor(Xi_d1) + factor(Xi_d2), regtype="lc", bwmethod = "cv.ls", data=data_ins)  
    nb.np <- npreg(bws = bw.nb)
    
    bw.object.DGP1[1,-1,i,j] <- bw.nb$bw
    
    pred <- predict(nb.np,  type="response", newdata=data_oos)
    
       # oracle MSE 
    intermediate_res.oMSE[i,8,j] <- 1/q*sum((data_oos$mu-pred)^2)
    # oracle MAE
    intermediate_res.oMAE[i,8,j] <- 1/q*sum(abs((data_oos$mu-pred)))
    # oracle RMSE
    intermediate_res.oRMSE[i,8,j] <- sqrt(1/q*sum((data_oos$mu-pred)^2))
    
    pred2 <- nb_sigma2(pred) # estimate sigma from LLNB model
    
    # oracle MSE, MAE, RMSE for sigma
    intermediate_res.oMSE.sigma[i,8,j] <- 1/q*sum((data_oos$size - pred2)^2)
    intermediate_res.oRMSE.sigma[i,8,j] <- sqrt(1/q*sum((data_oos$size - pred2)^2))
    intermediate_res.oMAE.sigma[i,8,j] <- 1/q*sum(abs(data_oos$size - pred2))
    
    # Bias
    intermediate_res.bias.y[i,8,j] <- 1/q*sum(pred - data_oos$Y)
    intermediate_res.bias.mu[i,8,j] <- 1/q*sum(pred - data_oos$mu)
    intermediate_res.bias.sigma[i,8,j] <- 1/q * sum(pred2 - data_oos$size)
    
    rm(pred2)
    rm(pred)
   
    
     # Nonparametric Conditional Density Estimator (NPCDE)
    
     bw.dens <- npcdensbw(ordered(Y) ~ Xi_c1 + factor(Xi_d1) + factor(Xi_d2), regtype="lc", bwmethod = "cv.ls", data=data_ins) 
     fully_np <- npcdens(bws=bw.dens)
    
     bw.object.DGP1[2,-1,i,j] <- bw.dens$xbw
     bw.object.DGP1[2,1,i,j] <- bw.dens$ybw

     val <- 0:(max(data$Y)+5)
     pred <- predict_modal_dens(fully_np, data_oos, range=val)  # predict modal value of distribution!
  
    # oracle MSE 
    intermediate_res.oMSE[i,9,j] <- 1/q*sum((data_oos$mu-pred)^2)
    # oracle MAE
    intermediate_res.oMAE[i,9,j] <- 1/q*sum(abs((data_oos$mu-pred)))
    # oracle RMSE
    intermediate_res.oRMSE[i,9,j] <- sqrt(1/q*sum((data_oos$mu-pred)^2))
    
    # Bias
    intermediate_res.bias.y[i,9,j] <- 1/q*sum(pred - data_oos$Y)
    intermediate_res.bias.mu[i,9,j] <- 1/q*sum(pred - data_oos$mu)
    
    rm(pred)
  
    
  }} 

end.time <- Sys.time()
time <- end.time - start.time   # compute running time!
time

### Number of convergence failures (PNB): 
conv.fail <- matrix(NA, ncol=7, nrow=length(n)+1)
colnames(conv.fail) <- c("PNB1", rep(NA,5), "PNB2")
rownames(conv.fail) <- c("100", "200", "400", "total")

for (j in 1:(dim(conv.fail)[2])){
    for (i in 1:length(n)){
  conv.fail[i,j] <- sum(is.na(intermediate_res.oMSE[i,j,]))
  conv.fail[length(n)+1,j] <- sum(is.na(intermediate_res.oMSE[,j,]))
  }
}

conv.fail

### Results 
results.oMSE <- mean_array(intermediate_res.oMSE)
results.oMAE <- mean_array(intermediate_res.oMAE)
results.oRMSE <- mean_array(intermediate_res.oRMSE)

results.oMSE.sigma <- mean_array(intermediate_res.oMSE.sigma)
results.oRMSE.sigma <- mean_array(intermediate_res.oRMSE.sigma)
results.oMAE.sigma <- mean_array(intermediate_res.oMAE.sigma)

results.bias.y <- mean_array(intermediate_res.bias.y)
results.bias.mu <- mean_array(intermediate_res.bias.mu)
results.bias.sigma <- mean_array(intermediate_res.bias.sigma)

# For sigma-results: exclude sigma estimates with convergence problems (both PNB and LLNB)
# Rule: exclude repetitions with oMSE(sigma) > 80

# create arrays for cleaned results
intermediate_res.oMSE.sigma2 <- intermediate_res.oMSE.sigma
intermediate_res.oRMSE.sigma2 <- intermediate_res.oRMSE.sigma
intermediate_res.oMAE.sigma2 <- intermediate_res.oMAE.sigma


# PNB
for (j in 1:R) { # over repetitions
  
  for (k in 1:(dim(intermediate_res.oMSE.sigma)[2])){ # for every model with estimated sigma2
    
  for (i in 1:(dim(intermediate_res.oMSE.sigma)[1])) { # over rows
    if  ( ( is.na(intermediate_res.oMSE.sigma[i,k,j]) == FALSE ) && (intermediate_res.oMSE.sigma[i,k,j] > 80) )
      intermediate_res.oMSE.sigma2[i,k,j] <- NA
    # if (is.na(intermediate_res.oMSE.sigma[i,2,j] ==TRUE) )
    #    intermediate_res.oMSE.sigma[i,2,j] <- NA
  }
  }
} 


# Exclude outliers according to the same (MSE-based) rule: MAE!
# (Outliers are easier to find based on MSE & comparability is guaranteed)

# PNB
for (j in 1:R) { # over repetitions
  
  for(k in 1:(dim(intermediate_res.oMSE.sigma)[2])){ # for every estimator
    
  for (i in 1:(dim(intermediate_res.oMSE.sigma)[1])) { # over rows
    if  ( ( is.na(intermediate_res.oMSE.sigma[i,k,j]) == FALSE ) && (intermediate_res.oMSE.sigma[i,k,j] > 80) )
      intermediate_res.oMAE.sigma2[i,k,j] <- NA
    # if (is.na(intermediate_res.oMSE.sigma[i,2,j] ==TRUE) )
    #    intermediate_res.oMSE.sigma[i,2,j] <- NA
  }
  } 
}


# Apply rule also for bias (sigma)
intermediate_res.bias.sigma2 <- intermediate_res.bias.sigma

# PNB
for (j in 1:R) { # over repetitions
  for(k in 1:(dim(intermediate_res.oMSE.sigma)[2])){ # for every estimator
    
    for (i in 1:(dim(intermediate_res.oMSE.sigma)[1])) { # over rows
    if  ( ( is.na(intermediate_res.oMSE.sigma[i,k,j]) == FALSE ) && (intermediate_res.oMSE.sigma[i,k,j] > 80) )
      intermediate_res.bias.sigma2[i,k,j] <- NA
    # if (is.na(intermediate_res.oMSE.sigma[i,2,j] ==TRUE) )
    #    intermediate_res.oMSE.sigma[i,2,j] <- NA
    }
  }
} 


# Update Results w.r.t sigma: 
results.oMSE.sigma <- mean_array(intermediate_res.oMSE.sigma2)
intermediate_res.oRMSE.sigma2 <- sqrt(intermediate_res.oMSE.sigma2)
results.oRMSE.sigma <- mean_array(intermediate_res.oRMSE.sigma2)
results.oMAE.sigma <- mean_array(intermediate_res.oMAE.sigma2)

results.bias.sigma <- mean_array(intermediate_res.bias.sigma2)


# For mu-results: exclude estimates with convergence problems (both PNB and LLNB)
# Rule: exclude repetitions with oMSE(mu) > 30
# Apply rule also for bias (mu)
intermediate_res.oMSE2 <- intermediate_res.oMSE


# PNB
for (j in 1:R) { # over repetitions
  for(k in 1:(dim(intermediate_res.oMSE)[2])){ # for every estimator
    
    for (i in 1:(dim(intermediate_res.oMSE)[1])) { # over rows
      if  ( ( is.na(intermediate_res.oMSE[i,k,j]) == FALSE ) && (intermediate_res.oMSE[i,k,j] > 30) )
        intermediate_res.oMSE2[i,k,j] <- NA
      # if (is.na(intermediate_res.oMSE[i,2,j] ==TRUE) )
      #    iintermediate_res.oMSE[i,2,j] <- NA
    }
  }
} 

### Results with correction of convergence problems
results.oMSE2 <- mean_array(intermediate_res.oMSE2)
intermediate_res.oRMSE2 <- sqrt(intermediate_res.oMSE2)
results.oRMSE2 <- mean_array(intermediate_res.oRMSE2)

### Names for result objects
colnames(results.oMSE2) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
rownames(results.oMSE2) <- c("100", "200", "400")
colnames(results.oRMSE2) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
rownames(results.oRMSE2) <-  c("100", "200", "400")

### Names for result objects
colnames(results.oMSE) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.oMAE) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.oRMSE) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")

colnames(results.bias.y) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.bias.mu) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(results.bias.sigma) <-c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")

rownames(results.oMSE) <-  c("100", "200", "400")
rownames(results.oMAE) <-  c("100", "200", "400")
rownames(results.oRMSE) <-  c("100", "200", "400")

rownames(results.bias.y) <-  c("100", "200", "400")
rownames(results.bias.mu) <-  c("100", "200", "400")
rownames(results.bias.sigma) <-  c("100", "200", "400")

results.oMSE
results.oMAE
results.oRMSE

results.bias.y
results.bias.mu


#### Intermediate Results: 
colnames(intermediate_res.oMSE) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")
colnames(intermediate_res.oMAE) <- c("PNB1", rep(NA,5), "PNB2", "LLNB", "NPCDE")

rownames(intermediate_res.oMSE) <- c("100", "200", "400")
rownames(intermediate_res.oMAE) <- c("100", "200", "400")

### bw objects: 
colnames(bw.object.DGP1) <- c("Y", "Xi_c1", "Xi_d1", "Xi_d2")
rownames(bw.object.DGP1) <- c("LLNB", "NPCDE")

#### Save bw object 
save(bw.object.DGP1, file="bw.object.DGP1.RData")

##### Transfer: R to Latex Output
## Generate and add index for DGP
DGP.ID <- matrix("DGP1",  nrow=length(n), ncol=1)
colnames(DGP.ID) <- c("DGP") 


############################################################
### Save Output with correction for convergence issues #####
############################################################

### Save final results as data (needed for stargazer package):
output2.DGP1 <- as.data.frame(cbind(DGP.ID, format(round(cbind(results.oMSE2, results.oMAE, results.oRMSE2, results.bias.y, results.bias.mu),4), nsmall=4)))
save(output2.DGP1,file="output2.DGP1.Rda")

# Parametric NB2 - linear model ("0")
output2.nb.DGP1 <-  as.data.frame(cbind(DGP.ID, format(round(cbind(results.bias.mu[,1],results.oMSE2[,1], results.oMAE[,1], results.oRMSE2[,1],  results.bias.sigma[,1], results.oMSE.sigma[,1], results.oMAE.sigma[,1], results.oRMSE.sigma[,1]),4), nsmall=4)))
colnames(output2.nb.DGP1) <-  cbind("DGP", "Bias", "oMSE", "oMAE", "oRMSE", "Bias", "oMSE", "oMAE", "oRMSE")
save(output2.nb.DGP1, file="output2.nb.DGP1.Rda")

# Parametric NB2 - PNB 1
output2.nb1.DGP1 <-  as.data.frame(cbind(DGP.ID, format(round(cbind(results.bias.mu[,2],results.oMSE2[,2], results.oMAE[,2], results.oRMSE2[,2],  results.bias.sigma[,2], results.oMSE.sigma[,2], results.oMAE.sigma[,2], results.oRMSE.sigma[,2]),4), nsmall=4)))
colnames(output2.nb1.DGP1) <-  cbind("DGP", "Bias", "oMSE", "oMAE", "oRMSE", "Bias", "oMSE", "oMAE", "oRMSE")
save(output2.nb1.DGP1, file="output2.nb1.DGP1.Rda")

# Parametric NB2 - PNB 2
output2.nb2.DGP1 <-  as.data.frame(cbind(DGP.ID, format(round(cbind(results.bias.mu[,7],results.oMSE2[,7], results.oMAE[,7], results.oRMSE2[,7],  results.bias.sigma[,7], results.oMSE.sigma[,7], results.oMAE.sigma[,7], results.oRMSE.sigma[,7]),4), nsmall=4)))
colnames(output2.nb2.DGP1) <-  cbind("DGP", "Bias", "oMSE", "oMAE", "oRMSE", "Bias", "oMSE", "oMAE", "oRMSE")
save(output2.nb2.DGP1, file="output2.nb2.DGP1.Rda")

# LLNB
output2.llnb.DGP1 <- as.data.frame(cbind(DGP.ID, format(round(cbind(results.bias.mu[,8],results.oMSE2[,8], results.oMAE[,8], results.oRMSE2[,8], results.bias.sigma[,8], results.oMSE.sigma[,8], results.oMAE.sigma[,8], results.oRMSE.sigma[,8]),4), nsmall=4)))
colnames(output2.llnb.DGP1) <- cbind("DGP", "Bias", "oMSE", "oMAE", "oRMSE", "Bias", "oMSE", "oMAE", "oRMSE")
save(output2.llnb.DGP1, file="output2.llnb.DGP1.Rda")

# NPCDE 
output2.npcde.DGP1 <- as.data.frame(cbind(DGP.ID, format(round(cbind(results.bias.mu[,9],results.oMSE2[,9], results.oMAE[,9], results.oRMSE2[,9]),4), nsmall=4)))
colnames(output2.npcde.DGP1) <- cbind("DGP", "Bias", "oMSE", "oMAE", "oRMSE")
save(output2.npcde.DGP1, file="output2.npcde.DGP1.Rda")


### Save intermediate results as data
intermed.oMSE.DGP1 <- intermediate_res.oMSE
intermed.oRMSE.DGP1 <- intermediate_res.oRMSE
intermed.oMAE.DGP1 <- intermediate_res.oMAE

save(intermed.oMSE.DGP1, file = "intermed.oMSE.DGP1.RData")
save(intermed.oRMSE.DGP1, file = "intermed.oRMSE.DGP1.RData")
save(intermed.oMAE.DGP1, file = "intermed.oMAE.DGP1.RData")

# Generate Boxplots of MSEs (intermediate Results)
# Boxplot Parametric Poisson, Parametric NB, (1) Local Likelihood Estimator, NPCDE

par(mfrow=c(1,1))
#Generate Boxplots for oMSE (mu) (intermediate Results)
    # Boxplot Parametric Poisson, Parametric NB, (1) Local Likelihood Estimator, NPCDE
    
    par(mfrow=c(1,1))
    # n = 100
    boxplot(intermediate_res.oMSE[1,1,], intermediate_res.oMSE[1,7,], intermediate_res.oMSE[1,8,], intermediate_res.oMSE[1,9,], names =  c("PNB (1)", "PNB (2)", "LLNB", "NPCDE"), ylim=c(0,5.2))
    title(main= "", sub = "n = 100")
    dev.copy(jpeg, filename="boxplot_mu_DGP1_1.jpg")
    dev.off()
    
    # n = 200
    boxplot(intermediate_res.oMSE[2,1,], intermediate_res.oMSE[2,7,], intermediate_res.oMSE[2,8,], intermediate_res.oMSE[2,9,], names =  c("PNB (1)", "PNB (2)", "LLNB", "NPCDE"), ylim=c(0,5.2))   
    title(sub = "n=200")
    dev.copy(jpeg, filename="boxplot_mu_DGP1_2.jpg")
    dev.off()
    
    # n = 400
    boxplot(intermediate_res.oMSE[3,1,], intermediate_res.oMSE[3,7,], intermediate_res.oMSE[3,8,], intermediate_res.oMSE[3,9,], names =  c("PNB (1)", "PNB (2)", "LLNB", "NPCDE"), ylim=c(0,5.2))
    title(sub = "n=400")
    dev.copy(jpeg, filename="boxplot_mu_DGP1_3.jpg")
    dev.off()

##### Boxplots with convergence correction
    par(mfrow=c(1,1))
    # n = 100
    boxplot(intermediate_res.oMSE2[1,1,], intermediate_res.oMSE2[1,7,], intermediate_res.oMSE2[1,8,], intermediate_res.oMSE2[1,9,], names = c("PNB (1)", "PNB (2)", "LLNB", "NPCDE"), ylim=c(0,4))
    title(main= "", sub = "n = 100")
    dev.copy(jpeg, filename="boxplot2_mu_DGP1_1.jpg")
    dev.off()
    
    # n = 200
    boxplot(intermediate_res.oMSE2[2,1,], intermediate_res.oMSE2[2,7,], intermediate_res.oMSE2[2,8,], intermediate_res.oMSE2[2,9,], names = c("PNB (1)", "PNB (2)", "LLNB", "NPCDE"), ylim=c(0,4))   
    title(sub = "n=200")
    dev.copy(jpeg, filename="boxplot2_mu_DGP1_2.jpg")
    dev.off()
    
    # n = 400
    boxplot(intermediate_res.oMSE2[3,1,], intermediate_res.oMSE2[3,7,], intermediate_res.oMSE2[3,8,], intermediate_res.oMSE2[3,9,], names = c("PNB (1)", "PNB (2)", "LLNB", "NPCDE"), ylim=c(0,4))
    title(sub = "n=400")
    dev.copy(jpeg, filename="boxplot2_mu_DGP1_3.jpg")
    dev.off()

# Save convergence failure results:
conv.fail.DGP1 <- conv.fail
save(conv.fail.DGP1, file="conv.fail.DGP1.RData")

# Save Workspace
save.image("workspaceDGP1.RData")


