clear all
set more off
//cd "C:\Users\farbma\Documents\Arbeit\AAA - Oregon Experiment Data\replication QJE"

//Finkelstein et al. only use $xs in their analysis of the survey data
global xs ddddraw_sur_2 ddddraw_sur_3 ddddraw_sur_4 ddddraw_sur_5 ddddraw_sur_6 ddddraw_sur_7 dddnumhh_li_2 dddnumhh_li_3 ddddraXnum_2_2 ddddraXnum_2_3 ddddraXnum_3_2 ddddraXnum_3_3 ddddraXnum_4_2 ddddraXnum_5_2 ddddraXnum_6_2 ddddraXnum_7_2
global addxs female_list hhinc_pctfpl_12m age2008 edu_12m_2 edu_12m_3 edu_12m_4 english_list zip_msa race_white_12m race_black_12m race_hisp_12m

//merge variables from "oregonhie_descriptive_vars.dta"
preserve
use "C:\Users\farbma\Documents\Arbeit\AAA - Oregon Experiment Data\replication QJE\Data\oregonhie_descriptive_vars.dta", clear
keep person_id birthyear_list
save "C:\Users\farbma\Documents\Arbeit\AAA - Oregon Experiment Data\replication QJE\Data\oregonhie_descriptive_vars_temp.dta", replace
restore
merge 1:1 person_id using "C:\Users\farbma\Documents\Arbeit\AAA - Oregon Experiment Data\replication QJE\Data\oregonhie_descriptive_vars_temp.dta"
assert _merge==3
drop _merge
erase "C:\Users\farbma\Documents\Arbeit\AAA - Oregon Experiment Data\replication QJE\Data\oregonhie_descriptive_vars_temp.dta"
gen age2008=2008-birthyear_list
drop birthyear_list

/////////////////////////////////////////////////////////////
//OLS estimation of Equation 1 in Finkelstein et al. QJE 2012
/////////////////////////////////////////////////////////////

gen chronicdis=dia_dx_12m+ast_dx_12m+hbp_dx_12m+emp_dx_12m+dep_dx_12m+ami_dx_12m+chf_dx_12m+chl_dx_12m+kid_dx_12m

//potential candidates for heterogeneity: 
codebook hhinc_pctfpl_12m, c 	// 1) 240 unique values
codebook age2008, c				// 2) 44 unique values
codebook chronicdis, c 			// 3) 10 unique values (number of chronic diseases)

//maybe also SNAP (food assistance) or TANF (welfare) benefits (in dollar!?!)
//parametric interactions
gen treat_hhinc_pct=treatment*hhinc_pctfpl_12m
gen treat_age=treatment*age2008
gen chronicdis_treat=treatment*chronicdis

//identifies those who participated in 12m survey
tab sample_12m_resp, m
tab sample_12m_resp, m nol

//additional socio-economic characteristics:
codebook female_list older younger english_list zip_msa, c
codebook edu_12m_1 edu_12m_2 edu_12m_3 edu_12m_4, c 
codebook fpl_categ_12m_1 fpl_categ_12m_2 fpl_categ_12m_3 fpl_categ_12m_4 fpl_categ_12m_5, c
codebook employ_hrs_12m_1 employ_hrs_12m_2 employ_hrs_12m_3 employ_hrs_12m_4, c 
codebook dia_dx_12m ast_dx_12m hbp_dx_12m emp_dx_12m dep_dx_12m, c			//reported in Table I
codebook ami_dx_12m chf_dx_12m chl_dx_12m kid_dx_12m, c						//not repored in Table I (additional chronic diseases!!!; introduction tells story about heterogeneous effects along type and number of chronic diseases)
codebook race_white_12m race_black_12m race_hisp_12m, c

//household income
sum hhinc_cat_12m hhinc_pctfpl_12m		//the latter one (percentage of FederalPovertyLevel) is probably "more" continuous; within one category the percentage of FPL is not the same!! so contains more information

//effect of lottery on "having no insurance at all"; kind of the denominator in the IV regression
tab ins_noins_12m if treatment==1
tab ins_noins_12m if treatment==0

//number of prescriptions; prim. care visits; ER visits, hospital visits
sum rx_num_mod_12m doc_num_mod_12m er_num_mod_12m hosp_num_mod_12m

//Replication of tables in QJE - Paper//
////////////////////////////////////////

//Table I (Panel A - all members of control sample): 
sum female_list older younger english_list zip_msa if treatment==0

//Table I (Panel B - members of control sample, who responded to survey): 
sum female_list older younger english_list zip_msa [aweight=weight_12m] if sample_12m_resp==1 & treatment==0
sum race_white_12m race_black_12m race_hisp_12m [aweight=weight_12m] if sample_12m_resp==1 & treatment==0
sum dia_dx_12m ast_dx_12m hbp_dx_12m emp_dx_12m dep_dx_12m [aweight=weight_12m] if sample_12m_resp==1 & treatment==0
sum ami_dx_12m chf_dx_12m dep_dx_12m chl_dx_12m kid_dx_12m [aweight=weight_12m] if sample_12m_resp==1 & treatment==0

//to restrict sample to N=23741 as in note of Table V 
keep if sample_12m_resp==1

//Table V: Health Care Utilization (Survey data) - perfectly replicates the results in Table V (p.1085)
reg rx_num_mod_12m treatment $xs [pweight=weight_12m], cluster(household_id)
reg doc_num_mod_12m treatment $xs [pweight=weight_12m], cluster(household_id)
gen fullsample=e(sample)
reg er_num_mod_12m treatment $xs [pweight=weight_12m], cluster(household_id)
reg hosp_num_mod_12m treatment $xs [pweight=weight_12m], cluster(household_id)

//our extended regression
reg doc_num_mod_12m treatment chronicdis $xs $addxs [pweight=weight_12m], cluster(household_id)
gen subsample=e(sample)

//regression as in Table V of Finkelstein but for our subsample 
reg doc_num_mod_12m treatment $xs [pweight=weight_12m] if subsample, cluster(household_id)

//descriptive statistics//
//////////////////////////

//fullsample
sum female_list race_white_12m race_black_12m race_hisp_12m english_list zip_msa younger older ///
    fpl_categ_12m_1 fpl_categ_12m_2 fpl_categ_12m_3 fpl_categ_12m_4 fpl_categ_12m_5 ///
	edu_12m_1 edu_12m_2 edu_12m_3 edu_12m_4 ins_any_12m ins_ohp_12m ins_privpay_12m ///
	dia_dx_12m ast_dx_12m hbp_dx_12m emp_dx_12m dep_dx_12m ami_dx_12m chf_dx_12m chl_dx_12m kid_dx_12m chronicdis ///
	doc_num_mod_12m er_num_mod_12m hosp_num_mod_12m rx_num_mod_12m if fullsample==1

	sum female_list race_white_12m race_black_12m race_hisp_12m english_list zip_msa younger older ///
    fpl_categ_12m_1 fpl_categ_12m_2 fpl_categ_12m_3 fpl_categ_12m_4 fpl_categ_12m_5 ///
	edu_12m_1 edu_12m_2 edu_12m_3 edu_12m_4 ins_any_12m ins_ohp_12m ins_privpay_12m ///
	dia_dx_12m ast_dx_12m hbp_dx_12m emp_dx_12m dep_dx_12m ami_dx_12m chf_dx_12m chl_dx_12m kid_dx_12m chronicdis ///
	doc_num_mod_12m er_num_mod_12m hosp_num_mod_12m rx_num_mod_12m if subsample==1

//parametric regressions:
nbreg doc_num_mod_12m treatment $xs [pweight=weight_12m] if subsample, cluster(household_id) disp(mean)
estat ic
*nbreg doc_num_mod_12m treatment chronicdis $xs $addxs [pweight=weight_12m] if subsample, cluster(household_id) disp(constant)
nbreg doc_num_mod_12m treatment $xs $addxs [pweight=weight_12m] if subsample, cluster(household_id) disp(mean)
estat ic
nbreg doc_num_mod_12m treatment chronicdis $xs $addxs [pweight=weight_12m] if subsample, cluster(household_id) disp(mean)
estat ic
nbreg doc_num_mod_12m treatment chronicdis chronicdis_treat $xs $addxs [pweight=weight_12m] if subsample, cluster(household_id) disp(mean)
estat ic

// no Weighting: 
nbreg doc_num_mod_12m treatment $xs, cluster(household_id) disp(mean)
estat ic
