################################################## 
################ Simulation Study ################ 
##################################################

#######################
# Auxiliary Functions #
#######################

# Function to compute conditional modal value of nonparametrically estimated density (NPCDE)
predict_modal_dens <- function(model, data, range) {
  #browser()
  n <- dim(data)[1]
  l <- length(range)
  range <- range
  aux.data <- data[rep(1:nrow(data),each=l),]
  aux.data[,"Y"] <- rep(range,n)
  pre <- predict(model, newdata=aux.data)
  modal <- vector(mode="numeric", length=n)
  for (i in 1:n) {
    ind <- c(((i-1)*l+1):(i*l))
    values <- pre[ind]
    values[is.nan(values)] <- -0.001
    ind_max <- which.max(values)
    ind_max <- ind_max+(i-1)*l
    modal[i] <- aux.data[ind_max,"Y"]
    if (sum(values)<=0) {modal[i] <- NA}
  }
  return(modal)
}


### Function to compute the average results over the R repetions
mean_array <- function(array) {
  dim <- dim(array)
  result <- matrix(NA, nrow=dim[1], ncol=dim[2])
  for (i in 1: dim[1]) {
    for (j in 1:dim[2]) {
      result[i,j] <- mean(array[i,j,], na.rm=TRUE)      
    }
  }
  return(result)
}


### Function that gives relative results to Parametric Poisson (DGP 1 & 2)
rel.PP <- function(matrix){
  dim <- dim(matrix)
  rel.res <- matrix(NA,  nrow=dim[1], ncol=dim[2])
  colnames(rel.res) <- colnames(matrix)
  rownames(rel.res) <- rownames(matrix)
  
  for (j in 1:dim[1]){
    for (i in 1:dim[2]){
      
      rel.res[j,i] <- (matrix[j,i]/matrix[j,1])*100
      rel.res <- round(rel.res,2)
      }
  }
  return(rel.res)  
}

#### Function that gives relative results to Parametric NB (DGP 3 & 4)
rel.PNB <- function(matrix){
  dim <- dim(matrix)
  rel.res <- matrix(NA,  nrow=dim[1], ncol=dim[2])
  colnames(rel.res) <- colnames(matrix)
  rownames(rel.res) <- rownames(matrix)
  
  for (j in 1:dim[1]){
    for (i in 1:dim[2]){
      
      rel.res[j,i] <- (matrix[j,i]/matrix[j,2])*100
      rel.res <- round(rel.res,2)
    }
  }
  return(rel.res)  
}



### Function to compute local likelihood estimator on sigma^(2) of LLNB
# cf. first order condition in Section 2 of Master Thesis
# Function for calculating log(Y!)

log_sum <- function(s) {
  n <- length(s)
  res <- vector(mode="numeric", length=n)
  for (i in 1:n) {
    if (s[i]==0)  {res[i] <- 0} else {res[i] <- sum(log(1:s[i]))}  
  }
  return(res)
}

# Function for calculating $\sum_{i=1}^{Y_i} log(\sigma^2+j-1)$

sum_sig <- function(y) {
  n <- length(y)
  res <- vector(mode="numeric", length=n)
  for (i in 1:n) {
    if (y[i]==0)  {res[i] <- 0} else {res[i] <- sum(log(sigma^(-2) + 0:(y[i]-1)))} }
  return(res)
}


nb_sigma2 <- function(lam_est) {
  y <- data_oos$Y
  n <- length(y)
  sig_est <- vector(mode="numeric", length=n) #estimated values
  #browser()
  for (i in 1:n) {
    #lambda <- lam_est[i]
    lambda <- lam_est
    Lik <- function(sigma) {
      ll <- sum(dnbinom(y,size=sigma, mu=lambda, log=T)*npksum(txdat=data_oos[,c(4,5,6)],bws=bw.nb$bw)$ksum)
      return(ll)
    }
    
    sig_est[i] <- optimize(Lik, lower=0, upper=10^8, maximum=TRUE)$maximum
      }
  
  return(sig_est)
}
